const fs = require('fs')
const MiniCssExtractPlugin = require('mini-css-extract-plugin')
// 文件或目录是否存在
const exists = exports.exists = function (path) {
    return fs.existsSync(path)
}

// 是否是文件
exports.isFile = function (path) {
    return exists(path) && fs.statSync(path).isFile()
}

// 是否是目录
exports.isDir = function (path) {
    return exists(path) && fs.statSync(path).isDirectory()
}

exports.stlyeRules = function stlyeRules (sourceMap, extract) {
    const types = [
      { test: /\.css$/, loader: 'css' },
      { test: /\.styl$/, loader: 'stylus' },
      { test: /\.less/, loader: 'less' },
      { test: /\.s[ac]ss$/, loader: 'sass' }]
    const options = { sourceMap: !!sourceMap }
    const use = [
      {
        loader: MiniCssExtractPlugin.loader,
        options: {}
      },
      {
        loader: 'css-loader',
        options: options
      },
      {
        loader: 'postcss-loader',
        options: Object.assign({
          plugins: (loader) => [
            require('autoprefixer')(), //CSS浏览器兼容
          ]
        }, options)
      }
    ]
  
    if (!extract) {
      use.unshift({
        loader: 'style-loader',
        options: options
      })
    }
  
    return types.map((t) => {
      const tuse = t.loader === 'css' ? use : use.concat({
        loader: t.loader + '-loader',
        options: options
      })
  
      return extract ? {
        test: t.test,
        use: tuse
      } : {
        test: t.test,
        use: tuse
      }
    })
  }
  
  exports.extractVueStlye = function extractVueStlye () {
    return {
      css: {
        use: [MiniCssExtractPlugin.loader, 'css-loader'],
        fallback: 'vue-style-loader'
      },
      stylus: {
        fallback: "vue-style-loader",
        use: [MiniCssExtractPlugin.loader, 'css-loader', 'stylus-loader']
      },
      less: {
        fallback: "vue-style-loader",
        use: [MiniCssExtractPlugin.loader, 'css-loader', 'less-loader']
      },
      sass: {
        fallback: "vue-style-loader",
        use: [MiniCssExtractPlugin.loader, 'css-loader', 'sass-loader']
      },
      scss: {
        fallback: "vue-style-loader",
        use: [MiniCssExtractPlugin.loader, 'css-loader', 'sass-loader']
      }
    }
  }